////////////////////////////////////////////////////////////////////////////////
// chaos_image_manager.h : image manager

#ifndef CHAOS_IMAGE_MANAGER_H_INCLUDED
#define CHAOS_IMAGE_MANAGER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// declare

class chaos_image_manager;

////////////////////////////////////////////////////////////////////////////////
// include

#ifndef CHAOS_POS_H_INCLUDED
#include "chaos_pos.h"
#endif // CHAOS_POS_H_INCLUDED

#ifndef CHAOS_RECT_H_INCLUDED
#include "chaos_rect.h"
#endif // CHAOS_RECT_H_INCLUDED

#ifndef CHAOS_COLOR_H_INCLUDED
#include "chaos_color.h"
#endif // CHAOS_COLOR_H_INCLUDED

#ifndef CHAOS_STRING_H_INCLUDED
#include "chaos_string.h"
#endif // CHAOS_STRING_H_INCLUDED

#ifndef CHAOS_IMAGE_H_INCLUDED
#include "chaos_image.h"
#endif // CHAOS_IMAGE_H_INCLUDED

#ifndef CHAOS_IMAGE_REALIZER_H_INCLUDED
#include "chaos_image_realizer.h"
#endif // CHAOS_IMAGE_REALIZER_H_INCLUDED

#ifndef CHAOS_MESSAGE_IMAGER_H_INCLUDED
#include "chaos_message_imager.h"
#endif // CHAOS_MESSAGE_IMAGER_H_INCLUDED

////////////////////////////////////////////////////////////////////////////////
// chaos_image_manager : image manager class

class chaos_image_manager {
// attribute
private:
	// screen ofs
	chaos_pos screen_ofs;
	// screen dim
	chaos_dim screen_dim;
	// main image
	chaos_image* main_image;
	// prompt image
	chaos_image* prompt_image;
	// screen image
	chaos_image* screen_image;
	// stretch blt info
	chaos_image::fast_smooth_stretch_blt_info* stretch_blt_info;
	//  image realizer(background)
	chaos_image_realizer* image_realizer;
	// message imager(text)
	chaos_message_imager* message_imager;
public:
	// get screen ofs
	chaos_pos get_screen_ofs() const {
		return screen_ofs;
	}
	// get screen dim
	chaos_dim get_screen_dim() const {
		return screen_dim;
	}

// create & destroy
public:
	// default constructor
	chaos_image_manager();
	// destructor
	~chaos_image_manager();
	// initialize
	bool initialize(
		chaos_dim sys_dim,
		const chaos_string& font_name,
		int font_size,
		bool font_bold,
		bool font_anti_alias,
		int image_buffer_count);
	// dispose
	void dispose();
	// change image buffer count
	bool change_image_buffer_count(
		int image_buffer_count);
	// change font
	bool change_font(
		const chaos_string& font_name,
		int font_size,
		bool font_bold,
		bool font_anti_alias);

// get & set
public:
	// get main image
	const chaos_image* get_main_image() const {
		return main_image;
	}
	chaos_image* get_main_image() {
		return main_image;
	}

// draw
public:
	// draw main image
	void draw_main_image(
		const chaos_string& info,
		const chaos_rect* sel_rect,
		chaos_color sel_color,
		bool prompt_visible, bool prompt_place,
		chaos_pos prompt_place_pos,
		chaos_rect* update_rect,
		const chaos_rect* clip_rect);

// message
public:
	// update message image
	void update_message_image(
		const chaos_string& message_text,
		chaos_rect* update_rect);
	// clear message image
	void clear_message_image();

// coordinate
public:
	// map coordinate
	static chaos_pos map_coordinate(
		chaos_pos dst_ofs,
		chaos_dim dst_dim,
		chaos_pos src_ofs,
		chaos_dim src_dim,
		chaos_pos src_pos,
		bool ceil);
	static chaos_rect map_coordinate(
		chaos_pos dst_ofs,
		chaos_dim dst_dim,
		chaos_pos src_ofs,
		chaos_dim src_dim,
		chaos_rect src_rect,
		bool outside);
	// map coordinate system to screen
	chaos_pos map_sys2screen(
		chaos_pos sys_pos,
		bool ceil);
	chaos_rect map_sys2screen(
		chaos_rect sys_rect,
		bool outside);
	// map coordinate screen to system
	chaos_pos map_screen2sys(
		chaos_pos screen_pos,
		bool ceil);
	chaos_rect map_screen2sys(
		chaos_rect screen_rect,
		bool outside);

// screen
public:
	// set frame rect
	void set_frame_rect(
		chaos_rect frame_rect);
	// get screen image
	const chaos_image* get_screen_image();
};

#endif // CHAOS_IMAGE_MANAGER_H_INCLUDED
